grammar Kasus;

options {
 output = AST;
 ASTLabelType =CommonTree;
 backtrack = true;
}

tokens {
	STATEMENT_LIST;
	DECLARATION;
	ASSIGNMENT;
	INPUT;	
	PRINT;		
	PRINTLN;	
	INT;
	STRING;
	ID;
	IF_STATEMENT;
	WHILE_STATEMENT;
}

// START:stat
prog:   declaration ';' stat+ -> ^(STATEMENT_LIST declaration stat+);

block 	: '{'
		stat*
	  '}' -> ^(STATEMENT_LIST stat*)
	;

stat:   assignment ';' -> assignment
    | ifstat
    | whilestat
    | input ';' -> input
    | print ';' ->print   
    | block 
    ;
    
 ifstat	:	'if' expr a=block ('else' b=block)? -> ^(IF_STATEMENT expr $a $b?)
    ;

whilestat
	:	 'while' expr block -> ^(WHILE_STATEMENT expr block)
	;

// END:stat

declaration
	:	
	'var' ID ( ',' ID)*   -> ^(DECLARATION ID*)
	;

print   :	
	'print' ID -> ^(PRINT ID)
	| 'print' STRING -> ^(PRINT STRING)
	| 'println' STRING -> ^(PRINTLN STRING)
	| 'println' ID -> ^(PRINTLN ID)		
	;
input	:	
	'input' ID -> ^(INPUT ID)
	;

assignment
	:			
	ID '=' expr -> ^(ASSIGNMENT ID expr)
	;
// START:expr

expr 	: condExpr;

condExpr:  aexpr (relationalOp^ aexpr)*
                ;

aexpr:   multExpr (('+'|'-')^ multExpr)*
    ; 

multExpr
    :   unaryExpr (('*'|'%'|'/')^ unaryExpr)*
    ; 


relationalOp 
    :    '<='
    |    '>='
    |   '<'
    |   '>'
    |    '=='
    |    '!='    
    ;
    
unaryExpr
	: unary_op^ atom
	| atom
	;    

atom:   INT 
    | ID
    |   '(' expr ')' -> expr
    ;

unary_op:	
	PLUS
	|MINUS
	;   
// END:expr

// START:tokens
INT :   '0'..'9'+ ;
ID:    'a'..'z'+ ;
WS  :   (' '|'\t'|'\n'|'\r')+ {skip();} ;
STRING	:	 '\''( ~('\'') )*'\'' ;
PLUS	:	 '+';
MINUS :	 '-';
// END:tokens
